import java.util.InputMismatchException;
import java.util.Scanner;

public class Connect4Game {
    private char[][] board;  
    private char currentPlayer;  
    private boolean gameOver;
    
    
    public Connect4Game() {
        board = new char[6][7];  
        currentPlayer = 'R';  
        gameOver = false;  
        
        for (int i = 0; i < 6; i++) {
            for (int j = 0; j < 7; j++) {
                board[i][j] = ' ';
            }
        }
    }
    
    public void playGame() {
        Scanner scanner = new Scanner(System.in);
        System.out.println(" ");
        System.out.println("    *** Connect 4 Game! *** ");
        boolean playAgain = true; 
        
        while (playAgain) { // Loop to play the game multiple times
            gameOver = false; // Reset gameOver for each playthrough
            printBoard();   
            try {
                while (!gameOver) {
                  System.out.print("Player " + currentPlayer + ", choose a column (0-6): ");
                  String input = scanner.nextLine();
                  if(input.isEmpty()) { // prevents game from freezing with empty space command
                    continue;
                  }
                  try {
                    if(input.equalsIgnoreCase("quit") || input.equalsIgnoreCase("q")) {
                        System.out.println(" ");
                        System.out.print("Are you sure you want to quit? (y/n): ");
                        String quitConfirmation = scanner.nextLine();
                        if(quitConfirmation.equalsIgnoreCase("y")) {
                            System.out.println(" ");
                            System.out.println("Thanks for playing! ");
                            System.out.println("Credits: Will DeKeizer");
                            System.out.println(" ");
                            System.exit(0);
                        } else {
                            System.out.println(" ");
                            continue;
                        }
                    }
                    
                    int column = Integer.parseInt(input);
                    if (isValidMove(column)) {
              
                      makeMove(column);
                      printBoard();
              
                      if (isWinningMove()) {
                       
                        System.out.println(" ");
                        System.out.println("Player " + currentPlayer + " wins!");
                        System.out.println(" ");
                        gameOver = true;
                      }
              
                      if (isBoardFull()) {
                        System.out.println(" ");  
                        System.out.println("It's a draw!");
                        System.out.println(" ");
                        gameOver = true;
                      }
                      currentPlayer = (currentPlayer == 'R') ? 'Y' : 'R';
                    } else {
                      System.out.println(" ");
                      System.out.println("Column is full, choose another");
                      System.out.println(" ");
                    }
              
                  } catch (NumberFormatException errorString) { // blocks user from inputing a word
                    System.out.println(" ");
                    System.out.println("Numbers 0-6 are only allowed as commands. Try again.");
                    System.out.println(" ");
                  }
                }

                System.out.print("Play again? (y/n): "); //prompts user if they want to play again
                String playAgainInput = scanner.next().toLowerCase();
                while (!playAgainInput.equals("y") && !playAgainInput.equals("n")); {
                
                
                       
                System.out.print("Invalid input. Play again? (y/n)"); //only y/n commands are accepted
                System.out.print("Play again? (y/n): ");
                playAgainInput = scanner.next().toLowerCase();
            }

            if (playAgainInput.equals("n")) {
                System.out.println("Thanks for playing! ");
                System.out.println("Credits: Will DeKeizer");
                playAgain = false; 
            } else {
                scanner.nextLine();
                resetGame();
            }
              } catch (InputMismatchException emptySpace) {
                scanner.nextLine();
              }
            
            
        }
        scanner.close();
    }
    
    private void resetGame() {
        currentPlayer = 'R';  
        gameOver = false;
        
        for (int i = 0; i < 6; i++) {
            for (int j = 0; j < 7; j++) {
                board[i][j] = ' ';
            }
        }
    }
    
    private void printBoard() {
        System.out.println(" ");
        System.out.println("   0   1   2   3   4   5   6  ");
        System.out.println(" ");
        for (int i = 0; i < 6; i++) {
            System.out.print(" | ");
            for (int j = 0; j < 7; j++) {
                System.out.print(board[i][j] + " | ");
            }
            System.out.println("\n");
        }
    }
    
    private boolean isValidMove(int column) {
        try {
            if (column < 0 || column > 7) {
                return false;
            }
            return board[0][column ] == ' ';
        } catch (ArrayIndexOutOfBoundsException f) {
            return false;
        }
    }
    
    private void makeMove(int column) {
        for (int i = 5; i >= 0; i--) {
            if (board[i][column] == ' ') {
                board[i][column] = currentPlayer;
                break;
            }
        }
    }
    
    private boolean isWinningMove() {
        for (int i = 0; i < 6; i++) { // Horizontal check
            for (int j = 0; j < 4; j++) {
                if (board[i][j] == currentPlayer && board[i][j+1] == currentPlayer &&
                    board[i][j+2] == currentPlayer && board[i][j+3] == currentPlayer) {
                    return true;
                }
            }
        }
    
        for (int i = 0; i < 3; i++) { // Vertical check
            for (int j = 0; j < 7; j++) {
                if (board[i][j] == currentPlayer && board[i+1][j] == currentPlayer &&
                    board[i+2][j] == currentPlayer && board[i+3][j] == currentPlayer) {
                    return true;
                }
            }
        }
        
        for (int i = 0; i < 3; i++) { // Descending diagonal check
            for (int j = 0; j < 4; j++) {
                if (board[i][j] == currentPlayer && board[i+1][j+1] == currentPlayer &&
                    board[i+2][j+2] == currentPlayer && board[i+3][j+3] == currentPlayer) {
                    return true;
                }
            }
        }
        
        for (int i = 3; i < 6; i++) { // Ascending diagonal check
            for (int j = 0; j < 4; j++) { 
                if (board[i][j] == currentPlayer && board[i-1][j+1] == currentPlayer &&
                    board[i-2][j+2] == currentPlayer && board[i-3][j+3] == currentPlayer) {
                    return true;
                }
            }
        }
        return false;
    }
    
    private boolean isBoardFull() {
        for (int i = 0; i < 6; i++) {
            for (int j = 0; j < 7; j++) {
                if (board[i][j] == ' ') {
                    return false;
                }
            }
        }
        return true;
    }  
    public static void main(String[] args) {
        Connect4Game game = new Connect4Game();
        game.playGame(); 
    }
}
